# encoding: utf-8

# Este arquivo foi criado para dar suporte a implementação de algumas estruturas
# de dados.
#
# Página: http://malbarbo.pro.br/ensino/2013/7256/
# Autor : Marco A L Barbosa <marco@malbarbo.pro.br>

import sys

if sys.version_info < (3,0):
    print('Voce esta usando uma versao antiga do Python')
    print('Instale a versão 3.3')
    sys.exit()

class Struct(object):
    '''
    Super classe para criar classe mais ricas. Classes derivadas de Struct
    podem ser comparadas por igualdade e exibidas no interpredador.  Para cada
    parâmetro no construtor é criado um atribudo. O método init deve ser
    chamado com estes parâmetros. Novos atributos podem ser criados, mas eles
    devem começar com _. Estes atribudos não são exibidos no interpretador.
    Exemplos
    >>> class A(Struct):
    ...     def __init__(self, x, y):
    ...         self.init(x, y)
    ...         self._z = x + y
    >>> p = A(4, 5)
    >>> p.x
    4
    >>> p.y
    5
    >>> p._z
    9
    >>> p.x = 20
    >>> p
    A(20, 5)
    >>> p == A(4, 5)
    False
    >>> p == A(20, 5)
    True
    >>> p == None
    False
    >>> p.w = 20
    Traceback (most recent call last):
        ...
    Exception: Campo inválido: w
    >>> class B(Struct):
    ...     def __init__(self, x, y):
    ...         self.init(x)
    >>> q = B(2, 3)
    Traceback (most recent call last):
        ...
    Exception: Número de argumentos inválido: esperava 2, mas obteve: 1
    '''
    def init(self, *args):
        Struct.__init__(self, *args)
   
    def __init__(self, *args):
        self._nome = self.__class__.__name__
        self._campos = self.__init__.__code__.co_varnames[1:]
        if len(args) != len(self._campos):
            raise Exception(
                'Número de argumentos inválido: esperava %d, mas obteve: %s' %
                 (len(self._campos), len(args)))
        for x, value in zip(self._campos, args):
            self.__setattr__(x, value)
           
    def __setattr__(self, name, value):
        if name.startswith('_') or name in self._campos:
            super().__setattr__(name, value)
        else:
            raise Exception('Campo inválido: ' + name)

    def _valores(self):
        return tuple(self.__getattribute__(x) for x in self._campos)

    def __repr__(self):
        return '%s(%s)' % (self._nome, ', '.join(map(repr, self._valores())))

    def __eq__(self, other):
        try:
            return self._valores() == other._valores()
        except:
            return False


def executa_testes(detalhes=False):
    '''
    Boolean -> None
    Executa os testes. Se detalhes for True, os detalhes dos testes são
    exibidos, caso contrário os detalhes são omitidos.
    '''
    import doctest
    import sys
    import os
    falhas, total = doctest.testmod(verbose=detalhes)
    if falhas == 0:
        print('%s: Passou em todos os %d testes' %
              (os.path.basename(sys.argv[0]), total))


if __name__ == '__main__':
    executa_testes()
